﻿using Otter;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Prehistech.Utility;

namespace Prehistech.Entities
{
    class CatStatue : Entity
    {
        private Spritemap<string> _spriteMap;
        private bool _hMove = false;
        private bool _vMove = false;
        private float _speed = 8.0f;
        private bool _isPoweredUp = false;
        private bool _isAngle = true;
        private bool _isJiggle = false;
        private bool _isJigglePlay = true;
        private bool _isLeap = false;
        private bool _isLeapPlay = true;
        private bool _isHover = false;
        private bool _isHoverPlay = true;
        private bool _isLand = false;
        private bool _isLandPlay = true;
        private double _angleInRadians = 1;
        private double _angleInDegrees = 1;
        private int hoverTimer = 30;
        private int catTimer = 30;
        private int CatTimerMax = 50;
        private bool onlyDieOnce = true;

        public CatStatue(float x, float y)
        {
            X = x;
            Y = y;
            SetHitbox(50, 140, (int)Global.Type.CATSTATUE);
            Collider.X = 70;
            Collider.Y = 50;
            _spriteMap = new Spritemap<string>(Assets.CATSTATUE, 200, 200);
            addNewSprites();
            _spriteMap.Play("poweredDown");
            Graphic = _spriteMap;
        }
        public override void Update()
        {
            base.Update();
            if (!_isPoweredUp)
            {
                var diff = Util.Distance(Global.PlayerX, Global.PlayerY, X + 50, Y + 50);

                if (Global.MoiDeaths > 1)
                {
                    _isPoweredUp = true;
                    _spriteMap.Play("stand");
                    SoundManager.PlayRequestedSound("_catStatuePowerUp");
                    SoundManager.PlayRequestedSound("_enemyStartRoar");
                }
            }
            if (catTimer <= 0)
            {
                catTimer = CatTimerMax;
                Global.GAME.Scene.Add(new Moi(Rand.Int(32, Global.MW), Rand.Int(32, Global.MH)));
            }
            if (catTimer > 0)
                catTimer--;
            if (_isPoweredUp)
            {
                if (_isAngle)
                {
                    GetPlayerAngle();
                    _isJiggle = true;
                }
                if (_isJiggle)
                {
                    if (_isJigglePlay)
                    {
                        _isJigglePlay = false;
                        _spriteMap.Play("jiggle");
                    }
                    X += 0;
                    Y += 0;
                    if (_spriteMap.CurrentAnim == "jiggle" && _spriteMap.CurrentFrameIndex == 10)
                    {
                        _isJigglePlay = true;
                        _isJiggle = false;
                        _isLeap = true;
                    }
                }
                if (_isLeap)
                {
                    if (_isLeapPlay)
                    {
                        _isLeapPlay = false;
                        _spriteMap.Play("leap");
                    }
                    MoveMoi();
                    if (_spriteMap.CurrentAnim == "leap" && _spriteMap.CurrentFrameIndex == 3)
                    {
                        _isLeapPlay = true;
                        _isLeap = false;
                        _isHover = true;
                    }
                }
                if (_isHover)
                {
                    if (_isHoverPlay)
                    {
                        _isHoverPlay = false;
                        _spriteMap.Play("hover");
                    }
                    MoveMoi();
                    if (_spriteMap.CurrentAnim == "hover" && hoverTimer <= 0)
                    {
                        hoverTimer = 30;
                        _isHoverPlay = true;
                        _isHover = false;
                        _isLand = true;
                    }
                    if (hoverTimer > 0)
                        hoverTimer--;
                }
                if (_isLand)
                {
                    if (_isLandPlay)
                    {
                        _isLandPlay = false;
                        _spriteMap.Play("land");
                    }
                    MoveMoi();
                    if (_spriteMap.CurrentAnim == "land" && _spriteMap.CurrentFrameIndex == 1)
                    {
                        _isLandPlay = true;
                        _isLand = false;
                        _isAngle = true;
                    }
                }
                if (Global.CatHealth <= 0)
                {
                    if (onlyDieOnce)
                    {
                        Global.MoiDeaths++;
                        onlyDieOnce = false;
                        _spriteMap.Play("death");
                        SoundManager.PlayRequestedSound("_catDeath");
                    }
                }
                #region Collisions
                var collb = Collider.Collide(X, Y, (int)Global.Type.BULLET);
                if ((collb != null || X < 0 || X >= Global.MW || Y < 0 || Y >= Global.MH))
                {
                    Global.CatHealth--;
                }
                var collr = Collider.Collide(X, Y, (int)Global.Type.BIGROCK);
                if ((collr != null) && _spriteMap.CurrentAnim != "death")
                {
                    Global.CatHealth -= 5;
                }
                #endregion
                if (_spriteMap.CurrentAnim == "death" && _spriteMap.CurrentFrameIndex == 17)
                    destroy();
            }
        }
        public void stopWalkingAnimations()
        {
            if (!_hMove && !_vMove)
            {
                switch (_spriteMap.CurrentAnim)
                {
                    case ("move"):
                        _spriteMap.Play("stand");
                        break;
                    default:
                        break;
                }
            }
        }
        public void MoveMoi()
        {
            X -= _speed * (float)Math.Cos(_angleInRadians);
            Y -= _speed * (float)Math.Sin(_angleInRadians);
        }
        public void DontMove()
        {
            X += 0;
            Y += 0;
        }
        public void GetPlayerAngle()
        {
            _isAngle = false;
            var vector1 = new Point((int)Global.PlayerX, (int)Global.PlayerY);
            var vector2 = new Point((int)X + 50, (int)Y + 50);
            _angleInRadians = Math.Atan2(vector2.Y - vector1.Y + 4, vector2.X - vector1.X + 4) + Rand.Float(.1f);
            if (_angleInRadians < 0) _angleInRadians += 2 * Math.PI;
            _angleInDegrees = MathHelper.ToDegrees((float)_angleInRadians);
        }
        public void addNewSprites()
        {
            var hMs = 6f;//horizontal animation speed
            _spriteMap.Add("stand", new int[] { 1 }, new float[] { hMs, hMs });
            _spriteMap.Add("poweredDown", new int[] { 0 }, new float[] { hMs, hMs });
            //i may have to break up the movement in to different parts. jump. whatever it is.
            _spriteMap.Add("jiggle", new int[] { 2, 3, 2, 3, 3, 2, 3, 3, 3, 2, 3 }, new float[] { 8f, 8f });
            _spriteMap.Add("leap", new int[] { 2, 3, 4, 5 }, new float[] { hMs, hMs });
            _spriteMap.Add("hover", new int[] { 5 }, new float[] { hMs, hMs });
            _spriteMap.Add("land", new int[] { 3, 1 }, new float[] { hMs, hMs });
            _spriteMap.Add("death", new int[] { 1, 2, 3, 2, 5, 6, 7, 7, 7, 8, 5, 1, 3, 2, 5, 0, 5, 3 }, new float[] { 10f, 10f });
        }
        public override void Render()
        {
            base.Render();
            //Collider.Render();
        }
        public void destroy()
        {
            Global.GAME.AddScene(new Prehistech.Scenes.WinScreen());
            RemoveSelf();
        }
    }
}
