﻿using Otter;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Prehistech.Utility;

namespace Prehistech.Entities
{
    class Moi : Entity
    {
        private Spritemap<string> _spriteMap;
        private bool _hMove = false;
        private bool _vMove = false;
        private float _speed = 2.0f;
        private bool _isPoweredUp = false;
        private bool _isAngle = true;
        private bool _isJiggle = false;
        private bool _isJigglePlay = true;
        private bool _isLeap = false;
        private bool _isLeapPlay = true;
        private bool _isHover = false;
        private bool _isHoverPlay = true;
        private bool _isLand = false;
        private bool _isLandPlay = true;
        private double _angleInRadians = 1;
        private double _angleInDegrees = 1;
        private int hoverTimer = 30;
        private bool _first = true;
        private bool onlyDieOnce = true;
        private int _moiHealth = 120;

        public Moi(float x, float y)
        {
            X = x;
            Y = y;
            SetHitbox(16, 46, (int)Global.Type.MOI);
            Collider.X = 15;
            Collider.Y = 9;
            _spriteMap = new Spritemap<string>(Assets.MOI, 46, 56);
            addNewSprites();
            _spriteMap.Play("poweredDown");
            Graphic = _spriteMap;
        }
        public override void Update()
        {
            base.Update();
            if (_first)
            {
                var collbu = Collider.Collide(X, Y, (int)Global.Type.BULLET);
                var colls = Collider.Collide(X, Y, (int)Global.Type.SOLID);
                var collp = Collider.Collide(X, Y, (int)Global.Type.PLAYER);
                var collm = Collider.Collide(X, Y, (int)Global.Type.MOI);
                if (collbu != null || colls != null || collp != null || collm != null || X < 0 || X > Global.MW || Y < 0 || Y > Global.MH)
                {
                    X = Otter.Rand.Int(32, Global.MW - 32);
                    Y = Otter.Rand.Int(32, Global.MH - 32);
                }
                else
                    _first = false;
            }
            else
            {
                if (!_isPoweredUp)
                {
                    var diff = Util.Distance(Global.PlayerX, Global.PlayerY, X, Y);
                    if (diff < 100)
                    {
                        _isPoweredUp = true;
                        _spriteMap.Play("stand");
                        SoundManager.PlayRequestedSound("_moiPowerUp", .8f);
                    }
                }
                if (_isPoweredUp)
                {
                    if (_isAngle)
                    {
                        GetPlayerAngle();
                        _isJiggle = true;
                    }
                    if (_isJiggle)
                    {
                        if (_isJigglePlay)
                        {
                            _isJigglePlay = false;
                            if (_moiHealth >= 100)
                                _spriteMap.Play("jiggle");
                            else if (_moiHealth >= 80)
                                _spriteMap.Play("jigglehurt1");
                            else if (_moiHealth >= 60)
                                _spriteMap.Play("jigglehurt2");
                            else if (_moiHealth >= 40)
                                _spriteMap.Play("jigglehurt3");
                            else if (_moiHealth >= 20)
                                _spriteMap.Play("jigglehurt4");
                            else if (_moiHealth > 0)
                                _spriteMap.Play("jigglehurt5");
                        }
                        X += 0;
                        Y += 0;
                        if (_spriteMap.CurrentAnim == "jiggle" && _spriteMap.CurrentFrameIndex == 10 ||
                            _spriteMap.CurrentAnim == "jigglehurt1" && _spriteMap.CurrentFrameIndex == 10 ||
                            _spriteMap.CurrentAnim == "jigglehurt2" && _spriteMap.CurrentFrameIndex == 10 ||
                            _spriteMap.CurrentAnim == "jigglehurt3" && _spriteMap.CurrentFrameIndex == 10 ||
                            _spriteMap.CurrentAnim == "jigglehurt4" && _spriteMap.CurrentFrameIndex == 10 ||
                            _spriteMap.CurrentAnim == "jigglehurt5" && _spriteMap.CurrentFrameIndex == 10)
                        {
                            _isJigglePlay = true;
                            _isJiggle = false;
                            _isLeap = true;
                        }
                    }
                    if (_isLeap)
                    {
                        if (_isLeapPlay)
                        {
                            _isLeapPlay = false;
                            if (_moiHealth >= 100)
                                _spriteMap.Play("leap");
                            else if (_moiHealth >= 80)
                                _spriteMap.Play("leaphurt1");
                            else if (_moiHealth >= 60)
                                _spriteMap.Play("leaphurt2");
                            else if (_moiHealth >= 40)
                                _spriteMap.Play("leaphurt3");
                            else if (_moiHealth >= 20)
                                _spriteMap.Play("leaphurt4");
                            else if (_moiHealth > 0)
                                _spriteMap.Play("leaphurt5");
                        }
                        MoveMoi();
                        if (_spriteMap.CurrentAnim == "leap" && _spriteMap.CurrentFrameIndex == 5 ||
                            _spriteMap.CurrentAnim == "leaphurt1" && _spriteMap.CurrentFrameIndex == 5 ||
                            _spriteMap.CurrentAnim == "leaphurt2" && _spriteMap.CurrentFrameIndex == 5 ||
                            _spriteMap.CurrentAnim == "leaphurt3" && _spriteMap.CurrentFrameIndex == 5 ||
                            _spriteMap.CurrentAnim == "leaphurt4" && _spriteMap.CurrentFrameIndex == 5 ||
                            _spriteMap.CurrentAnim == "leaphurt5" && _spriteMap.CurrentFrameIndex == 5)
                        {
                            _isLeapPlay = true;
                            _isLeap = false;
                            _isHover = true;
                        }
                    }
                    if (_isHover)
                    {
                        if (_isHoverPlay)
                        {
                            _isHoverPlay = false;
                            if (_moiHealth >= 100)
                                _spriteMap.Play("hover");
                            else if (_moiHealth >= 80)
                                _spriteMap.Play("hoverhurt1");
                            else if (_moiHealth >= 60)
                                _spriteMap.Play("hoverhurt2");
                            else if (_moiHealth >= 40)
                                _spriteMap.Play("hoverhurt3");
                            else if (_moiHealth >= 20)
                                _spriteMap.Play("hoverhurt4");
                            else if (_moiHealth > 0)
                                _spriteMap.Play("hoverhurt5");
                        }
                        MoveMoi();
                        if (_spriteMap.CurrentAnim == "hover" && hoverTimer <= 0 ||
                            _spriteMap.CurrentAnim == "hoverhurt1" && hoverTimer <= 0 ||
                            _spriteMap.CurrentAnim == "hoverhurt2" && hoverTimer <= 0 ||
                            _spriteMap.CurrentAnim == "hoverhurt3" && hoverTimer <= 0 ||
                            _spriteMap.CurrentAnim == "hoverhurt4" && hoverTimer <= 0 ||
                            _spriteMap.CurrentAnim == "hoverhurt5" && hoverTimer <= 0)
                        {
                            hoverTimer = 30;
                            _isHoverPlay = true;
                            _isHover = false;
                            _isLand = true;
                        }
                        if (hoverTimer > 0)
                            hoverTimer--;
                    }
                    if (_isLand)
                    {
                        if (_isLandPlay)
                        {
                            _isLandPlay = false;
                            if (_moiHealth >= 100)
                                _spriteMap.Play("land");
                            else if (_moiHealth >= 80)
                                _spriteMap.Play("landhurt1");
                            else if (_moiHealth >= 60)
                                _spriteMap.Play("landhurt2");
                            else if (_moiHealth >= 40)
                                _spriteMap.Play("landhurt3");
                            else if (_moiHealth >= 20)
                                _spriteMap.Play("landhurt4");
                            else if (_moiHealth > 0)
                                _spriteMap.Play("landhurt5");
                        }
                        MoveMoi();
                        if (_spriteMap.CurrentAnim == "land" && _spriteMap.CurrentFrameIndex == 5 ||
                            _spriteMap.CurrentAnim == "landhurt1" && _spriteMap.CurrentFrameIndex == 5 ||
                            _spriteMap.CurrentAnim == "landhurt2" && _spriteMap.CurrentFrameIndex == 5 ||
                            _spriteMap.CurrentAnim == "landhurt3" && _spriteMap.CurrentFrameIndex == 5 ||
                            _spriteMap.CurrentAnim == "landhurt4" && _spriteMap.CurrentFrameIndex == 5 ||
                            _spriteMap.CurrentAnim == "landhurt5" && _spriteMap.CurrentFrameIndex == 5)
                        {
                            _isLandPlay = true;
                            _isLand = false;
                            _isAngle = true;
                        }
                    }
                    if (_moiHealth <= 0)
                    {
                        if (onlyDieOnce)
                        {
                            Global.MoiDeaths++;
                            onlyDieOnce = false;
                            _spriteMap.Play("death");
                            if (ScreenX > 0 && ScreenX < 720 && ScreenY > 0 && ScreenY < 480)//the sound is on screen. yes, play it.
                                SoundManager.PlayRequestedSound("_moiDeath");
                        }
                    }
                    #region Collisions
                    var collb = Collider.Collide(X, Y, (int)Global.Type.BULLET);
                    if ((collb != null || X < 0 || X >= Global.MW || Y < 0 || Y >= Global.MH))
                    {
                        _moiHealth -= 2;
                    }
                    var collr = Collider.Collide(X, Y, (int)Global.Type.BIGROCK);
                    if ((collr != null) && _spriteMap.CurrentAnim != "death")
                    {
                        _moiHealth = 0;
                    }
                    if (_spriteMap.CurrentAnim == "death" && _spriteMap.CurrentFrameIndex == 17)
                        destroy();
                    #endregion
                }
            }
        }
        public void stopWalkingAnimations()
        {
            if (!_hMove && !_vMove)
            {
                switch (_spriteMap.CurrentAnim)
                {
                    case ("move"):
                        _spriteMap.Play("stand");
                        break;
                    default:
                        break;
                }
            }
        }
        public void MoveMoi()
        {
            X -= _speed * (float)Math.Cos(_angleInRadians);
            Y -= _speed * (float)Math.Sin(_angleInRadians);
        }
        public void DontMove()
        {
            X += 0;
            Y += 0;
        }
        public void GetPlayerAngle()
        {
            _isAngle = false;
            var vector1 = new Point((int)Global.PlayerX, (int)Global.PlayerY);
            var vector2 = new Point((int)X, (int)Y);
            _angleInRadians = Math.Atan2(vector2.Y - vector1.Y + 4, vector2.X - vector1.X + 4) + Rand.Float(.1f);
            if (_angleInRadians < 0) _angleInRadians += 2 * Math.PI;
            _angleInDegrees = MathHelper.ToDegrees((float)_angleInRadians);
        }
        public void addNewSprites()
        {
            var hMs = 6f;//horizontal animation speed
            _spriteMap.Add("stand", new int[] { 0 }, new float[] { hMs, hMs });
            _spriteMap.Add("poweredDown", new int[] { 8 }, new float[] { hMs, hMs });
            //i may have to break up the movement in to different parts. jump. whatever it is.
            _spriteMap.Add("jiggle", new int[] { 1, 2, 1, 2, 2, 1, 2, 2, 2, 1, 2 }, new float[] { hMs, hMs });
            _spriteMap.Add("leap", new int[] { 1, 2, 3, 4, 5, 6 }, new float[] { hMs, hMs });
            _spriteMap.Add("hover", new int[] { 7 }, new float[] { hMs, hMs });
            _spriteMap.Add("land", new int[] { 6, 5, 4, 3, 2, 1 }, new float[] { hMs, hMs });
            _spriteMap.Add("jigglehurt1", new int[] { 9, 32, 9, 32, 32, 9, 32, 32, 32, 9, 32 }, new float[] { hMs, hMs });
            _spriteMap.Add("leaphurt1", new int[] { 9, 32, 33, 34, 35, 36 }, new float[] { hMs, hMs });
            _spriteMap.Add("hoverhurt1", new int[] { 37 }, new float[] { hMs, hMs });
            _spriteMap.Add("landhurt1", new int[] { 36, 35, 34, 33, 32, 9 }, new float[] { hMs, hMs });
            _spriteMap.Add("jigglehurt2", new int[] { 10, 39, 10, 39, 39, 10, 39, 39, 39, 10, 39 }, new float[] { hMs, hMs });
            _spriteMap.Add("leaphurt2", new int[] { 10, 39, 40, 41, 42, 43 }, new float[] { hMs, hMs });
            _spriteMap.Add("hoverhurt2", new int[] { 44 }, new float[] { hMs, hMs });
            _spriteMap.Add("landhurt2", new int[] { 43, 42, 41, 40, 39, 10 }, new float[] { hMs, hMs });
            _spriteMap.Add("jigglehurt3", new int[] { 11, 46, 11, 46, 46, 11, 46, 46, 46, 11, 46 }, new float[] { hMs, hMs });
            _spriteMap.Add("leaphurt3", new int[] { 11, 46, 47, 48, 49, 50 }, new float[] { hMs, hMs });
            _spriteMap.Add("hoverhurt3", new int[] { 51 }, new float[] { hMs, hMs });
            _spriteMap.Add("landhurt3", new int[] { 50, 49, 48, 47, 46, 11 }, new float[] { hMs, hMs });
            _spriteMap.Add("jigglehurt4", new int[] { 12, 53, 12, 53, 53, 12, 53, 53, 53, 12, 53 }, new float[] { hMs, hMs });
            _spriteMap.Add("leaphurt4", new int[] { 12, 53, 54, 55, 56, 57 }, new float[] { hMs, hMs });
            _spriteMap.Add("hoverhurt4", new int[] { 58 }, new float[] { hMs, hMs });
            _spriteMap.Add("landhurt4", new int[] { 57, 56, 55, 54, 53, 12 }, new float[] { hMs, hMs });
            _spriteMap.Add("jigglehurt5", new int[] { 13, 66, 13, 66, 66, 13, 66, 66, 66, 13, 66 }, new float[] { hMs, hMs });
            _spriteMap.Add("leaphurt5", new int[] { 13, 66, 67, 68, 69, 70 }, new float[] { hMs, hMs });
            _spriteMap.Add("hoverhurt5", new int[] { 71 }, new float[] { hMs, hMs });
            _spriteMap.Add("landhurt5", new int[] { 70, 69, 68, 67, 66, 13 }, new float[] { hMs, hMs });
            _spriteMap.Add("death", new int[] { 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31 }, new float[] { hMs, hMs });
        }
        public override void Render()
        {
            base.Render();
            //Collider.Render();
        }
        public void destroy()
        {
            RemoveSelf();
        }
    }
}
