﻿using Otter;
using Prehistech.Utility;
using System;

namespace Prehistech.entities.bullets
{
    class GiantRockBall : Entity
    {
        public Spritemap<string> _spriteMap;
        private bool _first = true;
        private double _angleInRadians = 1;
        public static int AngleInDegrees = 1;
        private float _speed = 5.0f;

        public GiantRockBall(float x, float y)
        {
            X = x;
            Y = y;
            _spriteMap = new Spritemap<string>(Assets.BIGROCK, 100, 100);
            SetHitbox(70, 70, (int)Global.Type.BIGROCK);
            Collider.X = -35;
            Collider.Y = -35;
            _spriteMap.CenterOrigin();
            addNewSprites();
            _spriteMap.Play("shoot");
            Graphic = _spriteMap;
        }
        public override void Update()
        {
            base.Update();
            if (_first)//set the angle of the ball based on where the bullet starts! and the mouse.
            {
                _first = false;
                var vector1 = new Point((int)Scene.MouseX, (int)Scene.MouseY);
                var vector2 = new Point((int)X, (int)Y);
                _angleInRadians = Math.Atan2(vector2.Y - vector1.Y + 4, vector2.X - vector1.X + 4) + Rand.Float(.1f);
                if (_angleInRadians < 0) _angleInRadians += 2 * Math.PI;//turns the value into 2PI so we can get 360 degrees.
                AngleInDegrees = (int)MathHelper.ToDegrees((float)_angleInRadians);
                var collb = Collider.Collide(X, Y, (int)Global.Type.BIGROCK);
                if (collb != null || X < 0 || X >= Global.MW || Y < 0 || Y >= Global.MH)
                    destroy();
                else
                    SoundManager.PlayRequestedSound("_bigRockShot");
            }
            if (AngleInDegrees >= 90 && AngleInDegrees <= 256 && _spriteMap.CurrentAnim != "death")
                _spriteMap.Angle -= 10;
            if (AngleInDegrees < 90 || AngleInDegrees > 256 && _spriteMap.CurrentAnim != "death")
                _spriteMap.Angle += 10;
            if (_spriteMap.CurrentAnim != "death")
            {
                X -= _speed * (float)Math.Cos(_angleInRadians);
                Y -= _speed * (float)Math.Sin(_angleInRadians);
            }
            else
            {
                X += 0;//1f * (float)Math.Cos(_angleInRadians) + Rand.Float() * 2;
                Y += 0;//1f * (float)Math.Sin(_angleInRadians) + Rand.Float() * 2;
            }
            var colls = Collider.Collide(X, Y, (int)Global.Type.SOLID);
            if ((colls != null || X < 0 || X >= Global.MW || Y < 0 || Y >= Global.MH) && _spriteMap.CurrentAnim != "death")
            {
                _spriteMap.Play("death");
                _spriteMap.Angle = 0;
                if (ScreenX > 0 && ScreenX < 720 && ScreenY > 0 && ScreenY < 480)//the sound is on screen. yes, play it.
                    SoundManager.PlayRequestedSound("_bigRockDeath");
            }

            if (_spriteMap.CurrentAnim == "death" && _spriteMap.CurrentFrameIndex == 4)
                destroy();
        }

        public void addNewSprites()
        {
            _spriteMap.Add("shoot", new int[] { 1 }, new float[] { 5f, 5f });
            _spriteMap.Add("death", new int[] { 2, 3, 4, 5, 6 }, new float[] { 7f, 7f });
        }
        public override void Render()//for testing!
        {
            base.Render();
            //Collider.Render();
        }
        public void destroy()
        {
            //we are not doing a seperate death sprite. it will just play that animation and then remove itself. OR go into the 
            //pool of avaiable bullets. I haven't decided how i want to handle it. It's time to get smart on
            //how much we are removing and adding unnecsessary entities.
            //if we HAVE to get a larger sprite for deaths, then we can hotswap out the spritemaps.
            RemoveSelf();
        }
    }
}