﻿using Otter;
using Prehistech.Utility;
using System;

namespace Prehistech.entities.bullets
{
    class LeadBullet : Entity
    {
        public Spritemap<string> _spriteMap;
        private bool _first = true;
        private double _angleInRadians = 1;
        private float _speed = 4.0f;

        public LeadBullet(float x, float y)
        {
            X = x;
            Y = y;
            _spriteMap = new Spritemap<string>(Assets.LEADBULLET, 10, 10);
            SetHitbox(4, 4, (int)Global.Type.BULLET);
            Collider.X = 2.5f;
            Collider.Y = 2.5f;
            addNewSprites();
            _spriteMap.Play("shoot");
            Graphic = _spriteMap;
        }
        public override void Update()
        {
            base.Update();
            if (_first)//set the angle of the bullet based on where the bullet starts and the mouse.
            {
                _first = false;
                SoundManager.PlayRequestedSound("_leadBulletShot", .1f);
                var vector1 = new Point((int)Scene.MouseX, (int)Scene.MouseY);
                var vector2 = new Point((int)X, (int)Y);
                _angleInRadians = Math.Atan2(vector2.Y - vector1.Y + 4, vector2.X - vector1.X + 4) + Rand.Float(.1f);
                if (_angleInRadians < 0) _angleInRadians += 2 * Math.PI;//turns the value into 2PI so we can get 360 degrees.
                var angleInDegrees = MathHelper.ToDegrees((float)_angleInRadians);
            }
            if (_spriteMap.CurrentAnim != "death")
            {
                X -= _speed * (float)Math.Cos(_angleInRadians);
                Y -= _speed * (float)Math.Sin(_angleInRadians);
            }
            else
            {
                X += 10f * (float)Math.Cos(_angleInRadians) + Rand.Float() * 2;
                Y += 10f * (float)Math.Sin(_angleInRadians) + Rand.Float() * 2;
            }
            var colls = Collider.Collide(X, Y, (int)Global.Type.SOLID);
            var collm = Collider.Collide(X, Y, (int)Global.Type.MOI);
            if (colls != null || collm != null || X < 0 || X >= Global.MW || Y < 0 || Y >= Global.MH)
            {
                _spriteMap.Play("death");
                if (ScreenX > 0 && ScreenX < 720 && ScreenY > 0 && ScreenY < 480)//the sound is on screen. yes, play it.
                    SoundManager.PlayRequestedSound("_leadBulletDeath", .3f);
            }

            if (_spriteMap.CurrentAnim == "death" && _spriteMap.CurrentFrameIndex == 8)
                destroy();
        }

        public void addNewSprites()
        {
            _spriteMap.Add("shoot", new int[] { 0, 1, 2, 3 }, new float[] { 5f, 5f });
            _spriteMap.Add("death", new int[] { 4, 5, 6, 7, 8, 9, 10, 11, 12, 13 }, new float[] { 1f, 1f });
        }
        public override void Render()//for testing!
        {
            base.Render();
            //Collider.Render();
        }
        public void destroy()
        {
            //we are not doing a seperate death sprite. it will just play that animation and then remove itself. OR go into the 
            //pool of avaiable bullets. I haven't decided how i want to handle it. It's time to get smart on
            //how much we are removing and adding unnecsessary entities.
            //if we HAVE to get a larger sprite for deaths, then we can hotswap out the spritemaps.
            RemoveSelf();
        }
    }
}
