﻿using Otter;
using Prehistech;
using Prehistech.Entities;
using Prehistech.Utility;
using TiledSharp;

namespace Prehistech.Scenes
{
    class TempleMainRoom : Scene
    {
        public const int _width = Global.MW * 3;
        public const int _height = Global.MW * 2;

        public TmxMap tmxMap = new TmxMap(Assets.TEMPLEMAIN);
        public GridCollider grid = null;

        public Tilemap layerOne = new Tilemap(Assets.TILES, _width, _height, Global.GRID_WIDTH, Global.GRID_HEIGHT);
        public Tilemap layerTwo = new Tilemap(Assets.TILES, _width, _height, Global.GRID_WIDTH, Global.GRID_HEIGHT);
        public Tilemap layerThree = new Tilemap(Assets.TILES, _width, _height, Global.GRID_WIDTH, Global.GRID_HEIGHT);
        public Tilemap layerFour = new Tilemap(Assets.TILES, _width, _height, Global.GRID_WIDTH, Global.GRID_HEIGHT);

        //public Music music = new Music(Assets.LAB_MUSIC); TODO
        private bool _isPlaying = true;
        private bool _hMove = false;
        private bool _vMove = false;

        public TempleMainRoom()
        {
            #region MapCreation
            grid = new GridCollider(_width, _height, Global.GRID_WIDTH, Global.GRID_HEIGHT, (int)Global.Type.SOLID);
            Entity gridEntity = new Entity(0, 0, null, grid);
            Add(gridEntity);

            layerOne.DefaultLayerName = tmxMap.Layers[3].Name;
            layerTwo.DefaultLayerName = tmxMap.Layers[2].Name;
            layerThree.DefaultLayerName = tmxMap.Layers[1].Name;
            layerFour.DefaultLayerName = tmxMap.Layers[0].Name;

            for (int z = 0; z < tmxMap.Layers.Count; z++)
            {
                TmxLayer layer = tmxMap.Layers[z];

                if (layer.Name == "Layer 1")
                    layerOne.AddLayer(layer.Name, tmxMap.Layers.Count - z);
                else if (layer.Name == "Layer 2")
                    layerTwo.AddLayer(layer.Name, tmxMap.Layers.Count - z);
                else if (layer.Name == "Layer 3")
                    layerThree.AddLayer(layer.Name, tmxMap.Layers.Count - z);
                else
                    layerFour.AddLayer(layer.Name, tmxMap.Layers.Count - z);

                for (int x = 0; x < tmxMap.Width; x++)
                {
                    for (int y = 0; y < tmxMap.Height; y++)
                    {
                        int i = y * tmxMap.Width + x;
                        if (layer.Name == "Layer 1")//walls
                        {
                            if (layer.Tiles[i].Gid != 0)
                                layerOne.SetTile(x, y, layer.Tiles[i].Gid - 1, layer.Name);
                            if (layer.Tiles[i].Gid != 0)
                                grid.SetTile(x, y, true);
                        }
                        else if (layer.Name == "Layer 2")//main tiles
                        {
                            if (layer.Tiles[i].Gid != 0)
                                layerTwo.SetTile(x, y, layer.Tiles[i].Gid - 1, layer.Name);
                        }
                        else if (layer.Name == "Layer 3")//light blue water
                        {
                            if (layer.Tiles[i].Gid != 0)
                                layerThree.SetTile(x, y, layer.Tiles[i].Gid - 1, layer.Name);
                        }
                        else//dark blue water
                            layerFour.SetTile(x, y, layer.Tiles[i].Gid - 1, layer.Name);
                    }
                }
            }
            #endregion

            AddGraphic(layerFour);
            AddGraphic(layerThree);
            AddGraphic(layerTwo);
            AddGraphic(layerOne);

            Global.cam = new Prehistec.Utility.Camera(100, 2);
            Global.player = new Prehistech.Entities.Player(Global.MW / 2, Global.MH / 2);
            Add(Global.player);
            Add(Global.cam);
            SoundManager.PlayRequestedSound("_levelStarted");
            Add(new AssaultRifle(870, 870));
            Add(new BallCannon(880, 880));
            Add(new CatStatue(650, 500));
            for (var i = 0; i < 100; i++)
            {
                Add(new Moi(Otter.Rand.Int(16, Global.MW - 16), Otter.Rand.Int(16, Global.MH - 32)));
            }
            //Add(new Hearts(0, 0));
        }
        public override void Begin()
        {
            base.Begin();
            Global.cam.adjustToPlayer(Global.MW, Global.MH, Global.PlayerX, Global.PlayerY);
        }
        public override void Update()
        {
            base.Update();
            Global.cam.followPlayer(Global.MW, Global.MH, Global.PlayerX + 100, Global.PlayerY + 100, Global.PlayerSpeed);
            //moveLayers(layerOne, .000f);
            //moveLayers(layerTwo, .0003f);
            moveLayers(layerThree, .0002f);
            moveLayers(layerFour, .0001f);
            #region Mute Sounds
            if (Global.PlayerSession.Controller.Button("M").Pressed)
            {
                if (_isPlaying)
                {
                    SoundManager.isSoundEnabled = false;
                    _isPlaying = false;
                    //music.Pause();//todo
                }
                else
                {
                    SoundManager.isSoundEnabled = true;
                    _isPlaying = true;
                    //music.Play();//todo
                }
            }
            #endregion
            //Add(new Enemies(0, 400));//this was the UI for the enimies.
            #region Win Conditions
            #endregion

            #region Lose Conditions
            if (Global.PlayerHealth <= 0)
            {//TODO
                //Global.GAME.RemoveScene();
                // Global.GAME.AddScene(new LoseScreen());
            }
            #endregion
        }
        public override void Render()
        {
            base.Render();
            //grid.Render();
        }
        private void moveLayers(Tilemap map, float speed)
        {
            var btn = Global.PlayerSession.Controller;//i'm not typing this over and over.
            if (btn.Button("A").Down || btn.Button("Left").Down)
            {
                if (Global.PlayerX <= -16)
                    map.ScrollX -= 0;
                else
                    map.ScrollX -= speed;
                _hMove = true;
            }
            else if (btn.Button("D").Down || btn.Button("Right").Down)
            {
                if (Global.PlayerX >= Global.MW - 28)
                    map.ScrollX += 0;
                else
                    map.ScrollX += speed;
                _hMove = true;
            }
            else
                _hMove = false;
            if (btn.Button("W").Down || btn.Button("Up").Down)
            {
                if (Global.PlayerY <= -24)
                    map.ScrollY -= 0;
                else
                    map.ScrollY -= speed;
                _vMove = true;
            }
            else if (btn.Button("S").Down || btn.Button("Down").Down)
            {
                if (Global.PlayerY >= Global.MH - 42)
                    map.ScrollY += 0;
                else
                    map.ScrollY += speed;
                _vMove = true;
            }
            else
                _hMove = false;
            if (!_hMove && !_vMove)
            {
                map.ScrollX += 0;
                map.ScrollY += 0;
            }
        }
    }
}

